#ifndef _MAINDLG_CPP
#define _MAINDLG_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>

#include "../Resources/Resource.H"

#include "../Source/NSWFL.H"
#include "../Source/Debug.H"
#include "../Source/Entry.H"
#include "../Source/Routines.H"

#include "../Dialogs/MainDlg.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------(Variable Declarations)
HWND hMainDialog = NULL;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

__int64 GetCycleCount()
{
    __int64 iCycleCount = 0;

    __asm {
        rdtsc
        lea ecx, [iCycleCount]
        mov [ecx], eax
        mov [ecx+4], edx
    }

    return iCycleCount;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI CPUTimer_Thread(LPVOID lpVoid)
{
	HWND hWnd = (HWND) lpVoid;

    SYSTEM_INFO SI;

    GetSystemInfo(&SI);

	__int64 iBefore = 0;
	__int64 iAfter = 0;

	char sMHZ[32];
	char sMHZFrmt[32];

	while(true)
	{
		iBefore = GetCycleCount();
		Sleep(1000);
		iAfter = GetCycleCount();

		sprintf_s(sMHZ, sizeof(sMHZ), "%.2f", (((float)(iAfter - iBefore) / MHZ) * SI.dwNumberOfProcessors));

		FormatFloatString(sMHZFrmt, sizeof(sMHZFrmt), sMHZ);

		sprintf_s(sMHZ, sizeof(sMHZ), "%s MHz\n", sMHZFrmt);

		Set_Text(hWnd, sMHZ);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CALLBACK MainDialog(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HWND hTextStatic = NULL;

    static HMENU SystemMenu_hMenu = NULL;

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG) // Received A Initilize Dialog Message
    {
        hMainDialog = hWnd;

        SystemMenu_hMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(SystemMenu_hMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(SystemMenu_hMenu, MF_STRING, MAINDIALOG_MENU_ABOUT, "About");

        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)gsTitleCaption);
        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)));

        hTextStatic = GetDlgItem(hWnd, IDC_STATICTEXT);

        HDC hDC = GetDC(hTextStatic);
        //int xHeight = -MulDiv(24, GetDeviceCaps(hDC, LOGPIXELSY),72);

		ReleaseDC(hTextStatic, hDC);

        HFONT hFont = CreateFont( 24,
                    0,0,0,FW_NORMAL,0,0,0,
                    DEFAULT_CHARSET,
                    OUT_DEFAULT_PRECIS,
                    CLIP_DEFAULT_PRECIS,
                    DEFAULT_QUALITY,
                    DEFAULT_PITCH,
                    "Courier New"
        );

        SendMessage(hTextStatic, WM_SETFONT, (WPARAM)hFont, (LPARAM)TRUE);

		
		Set_Text(hTextStatic, "Processing...");

		CreateThread(NULL, 0, CPUTimer_Thread, (LPVOID)hTextStatic, 0, NULL);
		
		CenterWindow(hWnd);

        return TRUE; // Return TRUE to set the keyboard focus, Otherwise return FALSE.
    }

	//--------------------------------------------------------------------------

	if((HANDLE)lParam == hTextStatic)
	{
        return Set_Color(RGB(255,0,0), RGB(255,255,255), wParam);
	}

	//--------------------------------------------------------------------------

    if(uMsg == WM_COMMAND)
    {
        if(wParam == IDC_OK) //- Ok Button.
        {
            EndDialog(hWnd, 0);
            DestroyWindow(hWnd);

			hMainDialog = NULL;

            return TRUE;
        }

        return FALSE;
    }

    //--------------------------------------------------------------------------

	if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        //- Any painting should be done here.

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }

    //--------------------------------------------------------------------------

    if(uMsg == WM_SYSCOMMAND) //- Received a system menu message.
    {
        if(LOWORD(wParam) == MAINDIALOG_MENU_ABOUT) //- About.
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);
            return TRUE;
        }

        return FALSE;
    }

    //--------------------------------------------------------------------------

    if(uMsg == WM_CLOSE) //- Received close message.
    {
        EndDialog(hWnd,0);
        DestroyWindow(hWnd);

		hMainDialog = NULL;

        return TRUE;
    }

    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif

